package com.jsh.erp.controller.audit;


import com.jsh.erp.base.AjaxResult;
import com.jsh.erp.base.BaseController;
import com.jsh.erp.base.TableDataInfo;
import com.jsh.erp.datasource.dto.AuditDTO;
import com.jsh.erp.datasource.dto.AuditQueryDTO;
import com.jsh.erp.datasource.vo.AuditVo;
import com.jsh.erp.service.AuditService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.List;

@RestController
@RequestMapping(value = "/audit")
@Api(tags = {"审核接口"})
public class AuditController extends BaseController {

    @Resource
    private AuditService auditService;

    //待我审批列表
    @ApiOperation("待我审批列表")
    @PostMapping("/pendingApprovalList")
    public TableDataInfo pendingApprovalList(@RequestBody AuditQueryDTO auditQueryDTO){
        List<AuditVo> list = auditService.pendingApprovalList(auditQueryDTO);
        return getDataTable(list);
    }

    //待我审批列表
    @ApiOperation("待我审批数量")
    @GetMapping("/pendingApprovalCount")
    public AjaxResult pendingApprovalCount(){
        Long count = auditService.pendingApprovalCount();
        return AjaxResult.success(count);
    }

    //已处理审批列表
    @ApiOperation("已处理审批列表")
    @PostMapping("/processedApprovalList")
    public TableDataInfo processedApprovalList(@RequestBody AuditQueryDTO auditQueryDTO){
        List<AuditVo> list = auditService.processedApprovalList(auditQueryDTO);
        return getDataTable(list);
    }

    //我发起的任务列表
    @ApiOperation("我发起的任务列表")
    @PostMapping("/myApprovalList")
    public TableDataInfo myApprovalList(@RequestBody AuditQueryDTO auditQueryDTO){
        List<AuditVo> list = auditService.myApprovalList(auditQueryDTO);
        return getDataTable(list);
    }

    //审核历史
    @ApiOperation("审核历史")
    @GetMapping("/auditHistory")
    public AjaxResult auditHistory(@RequestParam(value = "id") Long id){
        return AjaxResult.success(auditService.getAuditHistory(id));
    }

    @ApiOperation("设置审核状态")
    @PostMapping("/setStatus")
    public AjaxResult setStatus(@RequestBody AuditDTO auditDTO, HttpServletRequest request) {
        boolean b = auditService.setStatus(auditDTO);
        return AjaxResult.success(b);
    }


}
