package com.jsh.erp.controller.audit;

import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.jsh.erp.base.AjaxResult;
import com.jsh.erp.base.BaseController;
import com.jsh.erp.base.TableDataInfo;
import com.jsh.erp.datasource.dto.AuditProcessDTO;
import com.jsh.erp.datasource.dto.AuditProcessQueryDTO;
import com.jsh.erp.datasource.entities.AuditProcess;
import com.jsh.erp.datasource.vo.AuditProcessVo;
import com.jsh.erp.query.LambdaQueryWrapperX;
import com.jsh.erp.service.AuditProcessService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.Arrays;
import java.util.List;

@RestController
@RequestMapping(value = "/auditProcess")
@Api(tags = {"审核流程接口"})
public class AuditProcessController extends BaseController {

    @Resource
    private AuditProcessService auditProcessService;

    @ApiOperation("审核流程列表")
    @PostMapping("/list")
    public TableDataInfo list(@RequestBody AuditProcessQueryDTO auditProcessQueryDTO){
        List<AuditProcessVo> list = auditProcessService.listBy(auditProcessQueryDTO);
        return getDataTable(list);
    }

    @ApiOperation("新增审核流程")
    @PostMapping("/add")
    public AjaxResult add(@RequestBody AuditProcessDTO auditProcessDTO) {
        AuditProcess auditProcess = auditProcessService.getOne(new LambdaQueryWrapperX<AuditProcess>().eq(AuditProcess::getType,auditProcessDTO.getType()).eq(AuditProcess::getDeleteFlag,false));
        if (auditProcess != null){
            return AjaxResult.error("该类型审核流程已存在");
        }
        boolean b = auditProcessService.add(auditProcessDTO);
        if (!b){
            return AjaxResult.error("创建失败，请联系系统管理员");
        }
        return AjaxResult.success();
    }

    @ApiOperation("修改审核流程及节点详情")
    @PostMapping("/detailUpdate")
    public AjaxResult detailUpdate(@RequestBody AuditProcessDTO auditProcessDTO) {
        boolean b = auditProcessService.detailUpdate(auditProcessDTO);
        if (!b){
            return AjaxResult.error("修改失败，请联系系统管理员");
        }
        return AjaxResult.success();
    }

    @ApiOperation(value = "删除审核流程")
    @DeleteMapping(value = "/delete")
    public AjaxResult deleteResource(@RequestParam("id") Long id) {
        auditProcessService.update(new UpdateWrapper<AuditProcess>().eq("id",id).set("delete_flag",true));
        return AjaxResult.success();
    }

    @ApiOperation(value = "批量删除审核流程")
    @DeleteMapping(value = "/deleteBatch")
    public AjaxResult batchDeleteResource(@RequestParam("ids") String ids) {
        String[] idArray = ids.split(",");
        Arrays.asList(idArray).forEach(id -> {
            auditProcessService.update(new UpdateWrapper<AuditProcess>().eq("id",id).set("delete_flag",true));
        });
        return AjaxResult.success();
    }

}
