package com.jsh.erp.service;

import com.alibaba.fastjson.JSONArray;
import com.baomidou.mybatisplus.extension.service.IService;
import com.jsh.erp.datasource.entities.MaterialCategory;
import com.jsh.erp.datasource.vo.TreeNode;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

public interface MaterialCategoryService extends IService<MaterialCategory> {

    /**
     * 根据id查询商品类别
     */
    MaterialCategory getMaterialCategory(long id);

    /**
     * 根据id集合查询商品类别 0
     */
    List<MaterialCategory> getMaterialCategoryListByIds(String ids)throws Exception;

    List<MaterialCategory> getMaterialCategory()throws Exception;

    /**
     * 根据商品类型ID查询全部子类型集合
     */
    List<MaterialCategory> getAllList(Long parentId);

    List<MaterialCategory> getMCList(Long parentId)throws Exception;

    List<MaterialCategory> select(String name, Integer parentId) throws Exception;

    /**
     * 新增商品类别
     * @param materialCategory 商品类别数据 0
     */
    @Transactional(value = "transactionManager", rollbackFor = Exception.class)
    int insertMaterialCategory(MaterialCategory materialCategory, HttpServletRequest request)throws Exception;

    /**
     * 修改商品类别
     * @param obj 商品类别数据 0
     */
    @Transactional(value = "transactionManager", rollbackFor = Exception.class)
    int updateMaterialCategory(MaterialCategory obj, HttpServletRequest request) throws Exception;

    @Transactional(value = "transactionManager", rollbackFor = Exception.class)
    int deleteMaterialCategory(Long id, HttpServletRequest request)throws Exception;

    @Transactional(value = "transactionManager", rollbackFor = Exception.class)
    int batchDeleteMaterialCategory(String ids, HttpServletRequest request)throws Exception;

    @Transactional(value = "transactionManager", rollbackFor = Exception.class)
    int batchDeleteMaterialCategoryByIds(String ids) throws Exception;

    /**
     * 检查类别名称是否存在 0
     */
    int checkIsNameExist(Long id, String name)throws Exception;


    /**
     * 根据类别id来查询类别信息 0
     * @param id 类别id
     */
    List<MaterialCategory> findById(Long id)throws Exception;

    /**
     * 获取商品类别树数据
     */
    /**
     * 获取除当前类型的商品树类型数据
     * @param id 当前商品类型id  id为空查询所有类型
     * @return 商品类别树数据
     */
    List<TreeNode> getMaterialCategoryTree(Long id);

    void  checkMaterialCategorySerialNo(MaterialCategory mc)throws Exception;

    /**
     * 根据名称获取类型 0
     * @param name
     */
    Long getCategoryIdByName(String name);

}
