package com.jsh.erp.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.jsh.erp.datasource.dto.AuditProcessDTO;
import com.jsh.erp.datasource.dto.AuditProcessQueryDTO;
import com.jsh.erp.datasource.entities.AuditProcess;
import com.jsh.erp.datasource.entities.User;
import com.jsh.erp.datasource.mappers.AuditNodeConfigMapper;
import com.jsh.erp.datasource.mappers.AuditProcessMapper;
import com.jsh.erp.datasource.vo.AuditProcessVo;
import com.jsh.erp.service.AuditProcessService;
import com.jsh.erp.service.UserService;
import com.jsh.erp.utils.PageUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

@Service
@Slf4j
public class AuditProcessServiceImpl extends ServiceImpl<AuditProcessMapper,AuditProcess> implements AuditProcessService {

    @Resource
    private AuditProcessMapper auditProcessMapper;
    @Resource
    private AuditNodeConfigMapper auditNodeConfigMapper;

    @Resource
    private UserService userService;

    /**
     * 查询审核流程列表
     * @param auditProcessQueryDTO 筛选条件
     */
    @Override
    public List<AuditProcessVo> listBy(AuditProcessQueryDTO auditProcessQueryDTO) {
        PageUtils.startPage();
        List<AuditProcessVo> list = auditProcessMapper.listBy(auditProcessQueryDTO);
        for (AuditProcessVo auditProcessVo : list) {
            auditProcessVo.setAuditNodes(auditNodeConfigMapper.listByProcessId(auditProcessVo.getId()));
        }
        return list;
    }

    /**
     * 新增审核流程
     * @param auditProcessDTO
     */
    @Override
    @Transactional(value = "transactionManager", rollbackFor = Exception.class)
    public boolean add(AuditProcessDTO auditProcessDTO) {
        try {
            User user = userService.getCurrentUser();
            auditProcessDTO.setCreateBy(user.getId());
            auditProcessDTO.setCreateTime(new Date());
            this.save(auditProcessDTO);
            auditProcessDTO.getAuditNodes().forEach(v -> {
                v.setProcessId(auditProcessDTO.getId());
                v.setCreateBy(user.getId());
                v.setCreateTime(new Date());
            });
            auditNodeConfigMapper.insertBatch(auditProcessDTO.getAuditNodes());
        }catch (Exception e){
            log.error("创建审核流程失败", e);
            return false;
        }
        return true;
    }

    /**
     * 修改审核流程及节点详情
     * @param auditProcessDTO
     */
    @Override
    @Transactional(value = "transactionManager", rollbackFor = Exception.class)
    public boolean detailUpdate(AuditProcessDTO auditProcessDTO) {
        try {
            this.updateById(auditProcessDTO);
            auditNodeConfigMapper.updateBatchById(auditProcessDTO.getAuditNodes());
        }catch (Exception e){
            log.error("修改审核流程失败", e);
            return false;
        }
        return true;
    }
}
