package com.jsh.erp.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.jsh.erp.datasource.dto.MaterialInputDto;
import com.jsh.erp.datasource.entities.AuditProcess;
import com.jsh.erp.datasource.entities.MaterialInput;
import com.jsh.erp.datasource.entities.User;
import com.jsh.erp.datasource.mappers.AuditProcessMapper;
import com.jsh.erp.datasource.mappers.MaterialInputMapper;
import com.jsh.erp.datasource.pda.dto.PDAGoodsInputDTO;
import com.jsh.erp.exception.ServiceException;
import com.jsh.erp.query.LambdaQueryWrapperX;
import com.jsh.erp.service.AuditService;
import com.jsh.erp.service.MaterialInputService;
import com.jsh.erp.service.SequenceService;
import com.jsh.erp.service.UserService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

@Service
@Slf4j
public class MaterialInputServiceImpl extends ServiceImpl<MaterialInputMapper,MaterialInput> implements MaterialInputService {

    @Resource
    private MaterialInputMapper materialInputMapper;
    @Resource
    private AuditProcessMapper auditProcessMapper;

    @Resource
    private SequenceService sequenceService;

    @Resource
    private AuditService auditService;

    @Resource
    private UserService userService;


    /**
     * PDA货物录入列表
     */
    @Override
    public List<MaterialInput> listBy(PDAGoodsInputDTO pdaInventoryDTO) {
        return materialInputMapper.listBy(pdaInventoryDTO);
    }

    /**
     * 添加商品录入信息
     * @param materialInput
     */
    @Override
    @Transactional(value = "transactionManager", rollbackFor = Exception.class)
    public boolean add(MaterialInput materialInput) {
        AuditProcess auditProcess = auditProcessMapper.selectOne(new LambdaQueryWrapperX<AuditProcess>().eq(AuditProcess::getType,"pda维护商品").eq(AuditProcess::getDeleteFlag,false));
        if (auditProcess == null){
            throw new ServiceException("pda录入商品当前没有审核流程，录入失败！");
        }
        try{
            //生成单据编号
            materialInput.setNumber("HWLR" + sequenceService.buildOnlyNumber());
            //设置创建时间
            materialInput.setCreateTime(new Date());
            //设置创建人
            User user = userService.getCurrentUser();
            materialInput.setCreateBy(user.getId());
            materialInputMapper.insert(materialInput);
            auditService.generateAuditByItemId(materialInput.getNumber(),"pda维护商品");
        }catch (Exception e){
            log.info("添加商品录入信息失败！",e);
            return false;
        }
        return true;
    }

    /**
     * 根据单据号查询商品录入信息详情
     * @param number 录入单号
     */
    @Override
    public MaterialInput getDetailByNumber(String number) {
        MaterialInput materialInput = this.getOne(new LambdaQueryWrapperX<MaterialInput>().eq(MaterialInput::getNumber,number).eq(MaterialInput::getDeleteFlag,false));
        return materialInput;
    }

    /**
     * 编辑货物信息录入
     * @param materialInputDto
     */
    @Override
    public boolean updateMaterialInput(MaterialInputDto materialInputDto) {
        try {
            this.updateById(materialInputDto);
        }catch (Exception e){
            log.info("修改货物信息录入失败！",e);
            return false;
        }
        return true;
    }

}
